from dataclasses import dataclass
from io import StringIO

import pandas as pd
import requests

from . import models
from .utils import _create_session, _create_url, cast_if_not_none


@dataclass
class DataSummaryProcessor:
    dataset: dict

    def process(self):
        base_data = self._base_data()
        return models.DatasetSummary(**base_data)

    def _base_data(self):
        return {
            "id": self.dataset["versioned_data_object_id"],
            "title": self.dataset["title"],
            "event_start": self.dataset["event_start"],
            "event_end": self.dataset["event_end"],
            "owner": self._get_owner(),
            "tags": self.dataset["tags"],
            "sport": self._get_sport(),
        }

    def _get_owner(self):
        owner_data = self.dataset["owner"]
        return models.User(
            id=owner_data["id"],
            first_name=owner_data["first_name"],
            last_name=owner_data["last_name"],
        )

    def _get_sport(self):
        sport_data = self.dataset["sport"]
        if sport_data is not None:
            return sport_data["name"]


@dataclass
class DatasetProcessor:
    response: requests.Response
    
    def __post_init__(self):
        self.data = self.response.json()
        self.data_type = self.data["versioned_data_object"]["data_type"]["data_type"]
        self.id = self.data["versioned_data_object"]["id"]

    def process(self):
        try:
            response_data, response_model = getattr(self, f"_process_{self.data_type}")()
        except AttributeError:
            response_data, response_model = self._process_unstructured()

        base_data = self._base_data()
        return response_model(
            response=self.response,
            type=self.data_type,
            **response_data,
            **base_data
        )

    def _base_data(self):
        return {
            "id": self.id,
            "title": self.data["versioned_data_object"]["metadatum"]["title"],
            "event_start": self.data["versioned_data_object"]["metadatum"]["event_start"],
            "event_end": self.data["versioned_data_object"]["metadatum"]["event_end"],
            "owner": self._get_owner(),
            "tags": self.data["versioned_data_object"]["metadatum"]["tags"],
        }

    def _get_owner(self):
        owner_data = self.data["versioned_data_object"]["metadatum"]["owner"]
        return models.User(
            id=owner_data["id"],
            first_name=owner_data["first_name"],
            last_name=owner_data["last_name"],
        )
    
    def _process_questionnaire_type(self):
        data_row = self.data["versioned_data_object"]["structured_data_objects"][0]["data_rows"][0]
        actual_data = data_row.pop("values")

        questions = []
        for key, value in actual_data.items():
            if not key.startswith("v"):
                # probably not a question
                # @TODO figure out if this is really the case
                continue
            questions.append(models.Question(question=key, answer=value))

        return {"questions": questions}, models.Questionnaire

    def _process_generic_csv_type(self):
        session = _create_session()
        response = session.get(
            url=_create_url(f"/data/{self.id}/download"),
            allow_redirects=True,
        )
        response.raise_for_status()

        dataframe = pd.read_csv(StringIO(response.content.decode('utf-8')))

        return {"dataframe": dataframe}, models.TabularData

    def _process_strava_type(self):
        columns = ["hr", "speed", "power", "cadence", "elevation", "latlong", "distance"]

        session = _create_session()
        response = session.get(
            url=_create_url(f"/data/{self.id}/time_series"),
            params={"keys": ",".join(columns)},
            allow_redirects=True,
        )
        response.raise_for_status()

        data = response.json()
        dataframe = pd.DataFrame(data["time_series"], index=data["offset"])

        return {"dataframe": dataframe}, models.TabularData

    def _process_polar_type(self):
        data_row = self.data["versioned_data_object"]["structured_data_objects"][0]["data_rows"][-1]

        response_data = {
            "calories": cast_if_not_none(data_row.get("calories", None), int),
            "steps": cast_if_not_none(data_row.get("active-steps", None), int),
        }

        return response_data, models.DailyActivity

    def _process_fitbit_type(self):
        data_row = self.data["versioned_data_object"]["structured_data_objects"][0]["data_rows"][0]

        response_data = {
            "calories": cast_if_not_none(data_row.get("calories", None), int),
            "steps": cast_if_not_none(data_row.get("steps", None), int),
            "distance": cast_if_not_none(data_row.get("distance", None), float),
            "floors": cast_if_not_none(data_row.get("floors", None), int),
            "resting_heart_rate": cast_if_not_none(data_row.get("resting_heart_rate", None), int),
            "minutes_sedentary": cast_if_not_none(data_row.get("minutes_sedentary", None), int),
            "minutes_lightly_active": cast_if_not_none(data_row.get("minutes_lightly_active", None), int),
            "minutes_fairly_active": cast_if_not_none(data_row.get("minutes_fairly_active", None), int),
            "minutes_very_active": cast_if_not_none(data_row.get("minutes_very_active", None), int),
            "sleep_duration": cast_if_not_none(data_row.get("sleep_minutes", None), int),
            "sleep_start": cast_if_not_none(data_row.get("sleep_start_time", None), int),
            "sleep_end": cast_if_not_none(data_row.get("sleep_get_up_time", None), int),
        }

        return response_data, models.DailyActivity

    def _process_unstructured(self):
        session = _create_session()
        response = session.get(
            url=_create_url(f"/data/{self.id}/download"),
            allow_redirects=True,
        )
        response.raise_for_status()

        return {"file_response": response}, models.UnstructuredData



import copy
from dataclasses import dataclass, fields
from datetime import datetime
from typing import List, Optional

import pandas as pd
import requests
from pydantic import BaseModel, Field, root_validator

from .raw import get_data


@dataclass
class User:
    id: int
    first_name: str
    last_name: str


@dataclass
class DatasetSummary:
    id: int
    event_start: datetime
    event_end: datetime
    owner: User
    sport: Optional[str]
    tags: List[str]
    title: Optional[str]
    
    def get_data(self):
        return get_data(self.id)


@dataclass
class BaseData:
    id: int
    event_start: datetime
    event_end: datetime
    owner: User
    tags: List[str]
    title: Optional[str]
    type: str
    response: requests.Response


@dataclass
class Question:
    question: str
    answer: str


@dataclass
class Questionnaire(BaseData):
    questions: List[Question]


@dataclass
class TabularData(BaseData):
    dataframe: pd.DataFrame


@dataclass
class DailyActivity(BaseData):
    steps: int = None
    distance: float = None
    calories: int = None
    floors: int = None
    sleep_start: datetime = None
    sleep_end: datetime = None
    sleep_duration: int = None
    resting_heart_rate: int = None
    minutes_sedentary: int = None
    minutes_lightly_active: int = None
    minutes_fairly_active: int = None
    minutes_very_active: int = None


@dataclass
class UnstructuredData(BaseData):
    file_response: requests.Response



from typing import Generator

from .models import DatasetSummary
from .utils import _create_session, _create_url
from .processing import DataSummaryProcessor


# TODO: host: app.sportdatavalley.nl/api/v1/dashboards/available_datasets


def my_datasets(limit: int = None) -> Generator[DatasetSummary, None, None]:
    """Generator function that returns dataset summaries s for the authenticated user.

    Args:
        limit: Maximum number of datasets to return.

    Yields:
        Yields dataset summaries as sdvclient.models.DatasetSummary
    """
    session = _create_session()
    num = 0
    page = 1
    while True:
        response = session.get(
            url=_create_url("/timeline/my_metadata"),
            params={"page": page},
            allow_redirects=False,
        )
        response.raise_for_status()

        datasets = response.json()["data"]

        if len(datasets) == 0:
            return

        for dataset in datasets:
            yield DataSummaryProcessor(dataset).process()

            num += 1
            if limit is not None and num >= limit:
                return

        page += 1


def network_datasets(limit: int = None, query: str = None) -> Generator[DatasetSummary, None, None]:
    """Generator function that returns dataset summaries s for the authenticated user.

    Args:
        limit: Maximum number of datasets to return.

    Yields:
        Yields dataset summaries as sdvclient.models.DatasetSummary
    """
    session = _create_session()
    num = 0
    page = 1
    while True:
        params= {"page": page}
        if query is not None:
            params["query"] = query

        response = session.get(
            url=_create_url("/timeline/network_metadata"),
            params=params,
            allow_redirects=False,
        )
        response.raise_for_status()

        datasets = response.json()["data"]

        if len(datasets) == 0:
            return

        for dataset in datasets:
            yield DataSummaryProcessor(dataset["metadatum"]).process()

            num += 1
            if limit is not None and num >= limit:
                return

        page += 1



import os

import requests

TOKEN_ENV_VAR = "SDV_AUTH_TOKEN"
BASE_URL_ENV_VAR = "SDV_URL"
API_PATH_ENV_VAR = "SDV_API_PATH"


def set_token(token):
    os.environ[TOKEN_ENV_VAR] = token


def _get_authorization_header():
    try:
        token =  os.environ[TOKEN_ENV_VAR]
        if token.lower().startswith("bearer"):
            return token
        else:
            return f"ApiToken {token}"
    except KeyError:
        raise ValueError(
            f"Env var {TOKEN_ENV_VAR} not set. Please set it by \`sdv.set_token(\"your token here\")`"
        )


def set_base_url(base_url):
    os.environ[BASE_URL_ENV_VAR] = base_url


def _get_base_url():
    try:
        return os.environ[BASE_URL_ENV_VAR]
    except KeyError:
        return "https://app.sportdatavalley.nl"


def set_api_path(api_path):
    os.environ[API_PATH_ENV_VAR] = api_path


def _get_api_path():
    try:
        return os.environ[API_PATH_ENV_VAR]
    except KeyError:
        return "/api/v1"


def _create_session():
    session = requests.Session()
    authorization_header = _get_authorization_header()
    session.headers.update({"Authorization": authorization_header})

    return session


def _create_url(resource):
    base_url = _get_base_url()
    api_path = _get_api_path()
    return f"{base_url}{api_path}{resource}"


def cast_if_not_none(value, to_type):
    if value is not None:
        return to_type(value)
    else:
        return value



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['sdvclient']

package_data = \
{'': ['*']}

install_requires = \
['pandas>=1.0.5,<2.0.0', 'pydantic>=1.5.1,<2.0.0', 'requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'sdvclient',
    'version': '0.4.0',
    'description': 'Sport Data Valley client library for Python',
    'long_description': '# Sport Data Valley client library for Python\n\n[![Downloads](https://pepy.tech/badge/sdvclient)](https://pepy.tech/project/sdvclient)\n\n## Introduction\n`sdvclient` is a Python client library for the Sport Data Valley platform.\nIt is basically a wrapper around the REST API (documented [here](https://app.sportdatavalley.nl/api-docs/index.html)).\n\n## Installation\nIf you are working in the Sport Data Valley JupyterHub environment, this library is automatically installed.\n\nIf you are working in a different Python environment, this library can be installed from [PyPI](https://pypi.org/project/sdvclient/):\n```bash\npip install sdvclient\n```\n\nWhen you have previously installed the library and want to upgrade to a newer version:\n```bash\npip install --upgrade sdvclient\n```\n\n## Usage\n```python\nimport sdvclient as sdv\n\n\nfor dataset in sdv.my_datasets():\n    # Do something\n    pass\n```\nThe dataset summaries that are returned from `my_datasets()` have attributes like `title`, `event_start`, `event_end`, `owner`, `sport`, `tags` and more...\n```python\ndataset.sport\n>>> "sports.riding"\n```\n\nTo retrieve data from your network:\n```python\n\nimport sdvclient as sdv\n\n\nfor dataset in sdv.network_datasets():\n    # Do something\n    pass\n```\n\n### Limit the number of results\nBoth `sdv.my_datasets()` and `sdv.network_datasets()` accept an optional `limit` argument that can be used to limit the number of dataset summaries that are returned.\n```python\n\nimport sdvclient as sdv\n\nfor dataset in sdv.my_datasets(limit=10):\n    # Process maximum 10 datasets\n    pass\n```\nPlease note that if there are less datasets available then the `limit` you specify, the number of returned dataset summaries is lower than `limit`.\n\n\n### Filter network data\n`sdv.network_datasets()` accepts an optional `query` argument that can be used to filter the returned datasets:\n```python\nimport sdvclient as sdv\n\nfor dataset in sdv.network_datasets(query="strava"):\n    # Process datasets that are matched by the "strava" query\n    pass\n```\nPlease note that the query argument filters on *all* the fields of a dataset.\nThis means that filtering on the name of a user does not necessarily only retrieve data for that user, as this name may also occur *anywhere* else in a different dataset.\n\nN.B. The `query` argument is **not** available for `sdv.my_datasets()`.\n\n### Retrieving raw/full data\nAfter you have retrieved a dataset summary, you can then continue to download the raw/full data from this dataset by calling the `get_data()` method on this object:\n```python\ndataset = dataset.get_data(id=1337)\n```\n\nEvery object that is returned from `get_data()` has attributes like `title`, `event_start`, `event_end`, `owner`, `sport`, `type`, `tags` and more fields depending on the data_type. For example a dataset with type `strava_type` has an attribute `dataframe` that contains a [pandas.DataFrame](https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.html) with the data from this dataset.\n```python\ndataset.data_type\n>>> "strava_type"\ndataset.dataframe\n>>> <pandas.DataFrame>\n```\n\n#### Strava data type\nAs mentioned above, dasets of type `strava_type` have an attribute dataframe with the corresponding data in a dataframe:\n```python\ndataset.data_type\n>>> "strava_type"\ndataset.dataframe\n>>> <pandas.DataFrame>\n```\n\n#### Questionnaire data type\nDatasets of type questionnaire have a `questions` attribute which contains of all the questions and answers in the questionnaire.\nFor each question+answer, the question and answer are available on the `question` and `answer` attributes, respectivily.\n```python\ndataset.questions[2].question\n>>> "this is a question"\n\ndataset.questions[2].answer\n>>> "this is an answer"\n```\n\n#### Generic CSV data type\nFor generic tabular data like csv\'s the returned dataset has an attribute dataframe with the corresponding data in a dataframe:\n```python\ndataset.data_type\n>>> "generic_csv_type"\ndataset.dataframe\n>>> <pandas.DataFrame>\n```\n\n#### Daily activity data type\nFor daily activity data that is coming from e.g. Fitbit or Polar, the returned dataset has a range of attributes:\n\n- steps\n- distance\n- calories\n- floors\n- sleep_start\n- sleep_end\n- sleep_duration\n- resting_heart_rate\n- minutes_sedentary\n- minutes_lightly_active\n- minutes_fairly_active\n- minutes_very_active\n\n```python\ndataset.data_type\n>>> "fitbit_type"\ndataset.resting_heart_rate\n>>> 58\n```\n\nPlease note that not all attributes are always available, this is platform and device dependent.\n\n\n#### Unstructured data\nUnstructured data is data (files) that Sport Data Valley does not know how to process.\nThese files are stored "as is" in the platform and can be download via this client library as well:\nFor generic tabular data like csv\'s the returned dataset has an attribute dataframe with the corresponding data in a dataframe:\nUnstructured data has a `file_response` attribute that contains a [requests.Response](https://requests.readthedocs.io/en/latest/api/#requests.Response) object.\n\n```python\ndataset.data_type\n>>> "unstructured"\ndataset.file_response\n>>> <Response [200]>\n```\n\nRead more about processing files downloaded with the Python requests library [here](https://requests.readthedocs.io/en/master/user/quickstart/).\nE.g. to process binary response content, see [here](https://requests.readthedocs.io/en/master/user/quickstart/#binary-response-content).\n\n\n#### Other data types\nAlthough this library will be updated when new data types are added it can happen that a specific data type is not fully supported yet. In that case the returned dateset will be identical as unstructured data, with an `file_response` attribute that contains a [requests.Response](https://requests.readthedocs.io/en/latest/api/#requests.Response) object.\nUnstructured data is data (files) that Sport Data Valley does not know how to process.\n\n```python\ndataset.data_type\n>>> "some new data type"\ndataset.file_response\n>>> <Response [200]>\n```\n\n\n### Authentication\nThe library retrieves your API token from the `SDV_AUTH_TOKEN` environment variable.\nIf you are working in the Sport Data Valley JupyterHub, this is automatically set.\nIf you are working in a different environment, you can retrieve an API token from the "Advanced" page [here](https://app.sportdatavalley.nl/profile/edit) and set it like this:\n\n```python\nsdv.set_token("your API token here")\n```\n\n\n\n## Development\n\n### Adding Python versions\nThe supported Python versions are specified in `pyproject.toml[tool.poetry.dependencies]#python`.\nThe Python versions that are tested are specified in `pyproject.toml[tool.tox]#envlist` and in `Dockerfile.test`.\nIf you want to add a new supported Python version, or want to test against a newer version of an existing Python version, the versions at these locations need to be updated.\n\n\n## Contributors\n- [Aart Goossens](https://twitter.com/aartgoossens)\n\n## License\nSee [LICENSE](LICENSE) file.\n',
    'author': 'Aart Goossens',
    'author_email': 'aart@sportdatavalley.nl',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://info.sportdatavalley.nl',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)



from .processing import DatasetProcessor
from .utils import _create_session, _create_url


def get_data(id: int):
    """Function that returns datasets by id.

    Args:
        id: De id of the dataset.

    Returns:
        Returns datasets as sdvclient.models.Dataset
    """
    session = _create_session()
    response = session.get(url=_create_url(f"/data/{id}"))
    response.raise_for_status()

    return DatasetProcessor(response).process()
